/*********************************************************************
This function searches a specified list for a specified item. If the 
target item is found, the function returns a list containing the position 
of item. If the item is found more than once, the function returns
a list containing all the indices of the target item. If the item is not 
found, the function returns an empty list. No restrictions exist on the 
types of items in the list.

Error Code Descriptions
90  -- Argument must be a list
930 -- Too few arguments
940 -- Too many arguments

Syntax: inlist(list, item)
        inlist(matrix, row)

Since a matrix is defined as a list of lists, this function will also
work with input such as inlist([1,2;2,3], {1,2})

Thanks to Samuel Stearley for his help in optimizing this program.

Brett Sauerwein
15 March 02
********************************************************************/

#define USE_TI89              
#define USE_TI92PLUS        
#define OPTIMIZE_ROM_CALLS   

#define RETURN_VALUE

#include <tigcclib.h>       

void _main(void)
{
	unsigned char *listPointer, *itemPointer;
	
	int argumentNumber = 0;
	int index = 1;
	
	// get a pointer to the first argument entered by the user
			
	InitArgPtr (listPointer);
	
	argumentNumber = remaining_element_count (listPointer);
	
	// throw the appropriate error if the user enters too many or
	// too few arguments, or doesn't enter a LIST or MATRIX
	
	if (argumentNumber > 2)
		ER_throwVar (940);
		
	if (argumentNumber < 2)
		ER_throwVar (930);
		
	if (*listPointer != LIST_TAG)
		ER_throwVar (90);
		
	// get a pointer to the second argument 
		
	itemPointer = next_expression_index (listPointer);
	
	// decrement listPointer so it points to first element in list
		
	listPointer--;
	
	// designate the end of the list of indices that will be
	// pushed to the expression stack
	
	push_quantum (END_TAG);
	
	// step through the list, examining each element to see
	// if it matches target item; if it does, add its
	// position to the list
	
	while (*listPointer != END_TAG)
	{
		// compare_expressions returns 0 if the items match
				
		if (compare_expressions(listPointer, itemPointer) == 0)
			push_longint (index);		
			
		// advance pointer to the next element
				
		listPointer = next_expression_index (listPointer);
		index++;					
	} 	
	
	// list on the top of the stack is in descending order, so
	// reverse the list so items are in ascending order
	
	push_reversed_tail (top_estack);  
	push_quantum (LIST_TAG);
	
	// estack clean up
	
	itemPointer = next_expression_index (top_estack);
	
	while (*itemPointer != END_TAG)
	{
		listPointer = itemPointer;
		itemPointer = next_expression_index (itemPointer);
		delete_expression (listPointer);
	}
	
} // main
